@@ -4,7 +4,7 @@ methods that act upon a transfer function to gain functions such as the
 mass function.
 '''
 
-version = '1.1.4'
+version = '1.1.5'
 
 ###############################################################################
 # Some Imports
@@ -18,6 +18,7 @@ from numpy import sin, cos, tan, abs, arctan, arccos, arcsin, exp
 
 import cosmography
 import tools
+from fitting_functions import fits
 
 ###############################################################################
 # The Class
@@ -178,7 +179,7 @@ class Perturbations(object):
                                "omegav"   : 0.728,
                                "omegan"   : 0.0,
                                "H0"       : 70.4,
-                               'cs2_lam' : 0,
+                               'cs2_lam' : 1,
                                'TCMB'     : 2.725,
                                'yhe'      : 0.24,
                                'Num_Nu_massless' : 3.04,
@@ -214,25 +215,9 @@ class Perturbations(object):
                                  'transfer__kmax':2}
 
 
-        #A dictionary of available HMF fitting functions and their identifiers
-        self.mf_fits = {
-            "PS":self._nufnu_PS,
-            "ST":self._nufnu_ST,
-            "Warren":self._nufnu_Warren,
-            "Jenkins":self._nufnu_Jenkins,
-            "Reed03":self._nufnu_Reed03,
-            "Reed07":self._nufnu_Reed07,
-            "Angulo":self._nufnu_Angulo,
-            "Angulo_Bound":self._nufnu_Angulo_Bound,
-            "Tinker":self._nufnu_Tinker,
-            "Watson_FoF":self._nufnu_Watson_FoF,
-            "Watson":self._nufnu_Watson,
-            "Crocce":self._nufnu_Crocce,
-            "Courtin":self._nufnu_Courtin,
-            "Bhattacharya": self._nufnu_Bhattacharya,
-            "Behroozi": self._nufnu_Tinker,
-            "user_model":self._nufnu_user_model
-            }
+        #A list of available HMF fitting functions and their identifiers
+        self.mf_fits = ["PS", "ST", "Warren", "Jenkins", "Reed03", "Reed07", "Angulo", "Angulo_Bound", "Tinker",
+                        "Watson_FoF", "Watson", "Crocce", "Courtin", "Bhattacharya", "Behroozi", "user_model"]
 
         self.update(M=M, mf_fit=mf_fit, k_bounds=k_bounds, transfer_file=transfer_file, wdm_mass=wdm_mass,
                      delta_vir=delta_vir, user_fit=user_fit, z=z, transfer_fit=transfer_fit, cut_fit=cut_fit, ** kwargs)
@@ -683,9 +668,9 @@ class Perturbations(object):
             return self.__unnormalized_power
         except:
             if self.transfer_fit == "CAMB":
-                self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_function_callable(self.lnk)
+                self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_function_callable(self.lnk) + np.log(2 * np.pi ** 2)
             else:
-                self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_original[1, :]
+                self.__unnormalized_power = self.cosmo_params['n'] * self.lnk + 2.0 * self._transfer_original[1, :] + np.log(2 * np.pi ** 2)
             return self.__unnormalized_power
     @_unnormalized_power.deleter
     def _unnormalized_power(self):
@@ -841,6 +826,7 @@ class Perturbations(object):
         try:
             del self.__sigma
             del self.fsigma
+            del self.lnsigma
         except:
             pass
 
@@ -889,9 +875,8 @@ class Perturbations(object):
         try:
             return self.__fsigma
         except:
-
-            nufnu = self.mf_fits[self.mf_fit]
-            #This is a little ambiguous, but nufnu could depend on: sigma, overdensity, delta_c, lnsigma
+            fits_class = fits(self.M, self.n_eff, self.mf_fit, self.sigma, self.cosmo_params['delta_c'], self.z, self.delta_vir, self.user_fit, self.cut_fit)
+            nufnu = fits_class.nufnu()
             self.__fsigma = nufnu()
             return self.__fsigma
     @fsigma.deleter
@@ -960,40 +945,101 @@ class Perturbations(object):
         The differential mass function in terms of natural log of M
         """
         try:
-            return self.__dndlnm
+            return self.__dndlog10m
         except:
-            self.__dndlnm = self.M * self.dndm * np.log(10)
-            return self.__dndlnm
+            self.__dndlog10m = self.M * self.dndm * np.log(10)
+            return self.__dndlog10m
 
     @dndlog10m.deleter
     def dndlog10m(self):
         try:
-            del self.__dndlnm
+            del self.__dndlog10m
         except:
             pass
 
-    def _ngtm(self):
-        ngtm = np.zeros_like(self.dndlnm)
+    def _upper_ngtm(self, M, mass_function, cut):
+        ### WE CALCULATE THE MASS FUNCTION ABOVE THE COMPUTED RANGE ###
+        # mass_function is logged already (not log10 though)
+        m_upper = np.linspace(np.log(M[-1]), np.log(10 ** 18), 500)
+        if cut:  #since its been cut, the best we can do is a power law
+            mf_func = spline(np.log(M), mass_function, k=1)
+            mf = mf_func(m_upper)
+        else:
+            #We try to calculate the hmf as far as we can normally
+            sigma_0 = tools.mass_variance(np.exp(m_upper), self._power_0, self.lnk, self.cosmo_params['mean_dens'])
+            sigma = sigma_0 * self.growth
+            dlnsdlnm = tools.dlnsdlnm(np.exp(m_upper), sigma_0, self._power_0, self.lnk, self.cosmo_params['mean_dens'])
+            n_eff = tools.n_eff(dlnsdlnm)
+            fsigma = fits(m_upper, n_eff, self.mf_fit, sigma, self.cosmo_params['delta_c'],
+                          self.z, self.delta_vir, self.user_fit, cut_fit=True).nufnu()()
+            #fsigma = nufnu()
+            dndm = fsigma * self.cosmo_params['mean_dens'] * np.abs(dlnsdlnm) / np.exp(m_upper) ** 2
+            mf = np.log(np.exp(m_upper) * dndm)
 
+            if np.isnan(mf[-1]):  #Then we couldn't get up all the way, so have to do linear ext.
+                if np.isnan(mf[1]):  #Then the whole extension is nan and we have to use the original (start at 1 because 1 val won't work either)
+                    mf_func = spline(np.log(M), mass_function, k=1)
+                    mf = mf_func(m_upper)
+                else:
+                    mfslice = mf[np.logical_not(np.isnan(mf))]
+                    m_nan = m_upper[np.isnan(mf)]
+                    m_true = m_upper[np.logical_not(np.isnan(mf))]
+                    mf_func = spline(m_true, mfslice, k=1)
+                    mf[len(mfslice):] = mf_func(m_nan)
+        return m_upper, mf
+
+    def _lower_ngtm(self, M, mass_function, cut):
+        ### WE CALCULATE THE MASS FUNCTION BELOW THE COMPUTED RANGE ###
+        # mass_function is logged already (not log10 though)
+        m_lower = np.linspace(np.log(10 ** 3), M[0], 500)
+        if cut:  #since its been cut, the best we can do is a power law
+            mf_func = spline(np.log(M), mass_function, k=1)
+            mf = mf_func(m_lower)
+        else:
+            #We try to calculate the hmf as far as we can normally
+            sigma_0 = tools.mass_variance(np.exp(m_lower), self._power_0, self.lnk, self.cosmo_params['mean_dens'])
+            sigma = sigma_0 * self.growth
+            dlnsdlnm = tools.dlnsdlnm(np.exp(m_lower), sigma_0, self._power_0, self.lnk, self.cosmo_params['mean_dens'])
+            n_eff = tools.n_eff(dlnsdlnm)
+            fsigma = fits(m_lower, n_eff, self.mf_fit, sigma, self.cosmo_params['delta_c'],
+                          self.z, self.delta_vir, self.user_fit, cut_fit=True).nufnu()()
+            #fsigma = nufnu()
+            dndm = fsigma * self.cosmo_params['mean_dens'] * np.abs(dlnsdlnm) / np.exp(m_lower) ** 2
+            mf = np.log(np.exp(m_lower) * dndm)
+
+            if np.isnan(mf[0]):  #Then we couldn't go down all the way, so have to do linear ext.
+                mfslice = mf[np.logical_not(np.isnan(mf))]
+                m_nan = m_lower[np.isnan(mf)]
+                m_true = m_lower[np.logical_not(np.isnan(mf))]
+                mf_func = spline(m_true, mfslice, k=1)
+                mf[:len(mfslice)] = mf_func(m_nan)
+        return m_lower, mf
+
+    def _ngtm(self):
+        """
+        This function is separated from the property because of the Behroozi fit
+        """
         # set M and mass_function within computed range
         M = self.M[np.logical_not(np.isnan(self.dndlnm))]
-        mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
+        mass_function = self.dndlnm[np.logical_not(np.isnan(self.dndlnm))]
 
-        # Interpolate the mass_function - this is in log-log space.
-        mf = spline(np.log(M), mass_function, k=1)
+        #Calculate the mass function (and its integral) from the highest M up to 10**18
+        if M[-1] < 10 ** 18:
+            m_upper, mf = self._upper_ngtm(M, np.log(mass_function), M[-1] < self.M[-1])
 
-        # Define max_M as either 18 or the maximum set by user
-        max_M = np.log(np.max([10 ** 18, M[-1]]))
+            int_upper = intg.simps(np.exp(mf), dx=m_upper[2] - m_upper[1], even='first')
+        else:
+            int_upper = 0
 
-        for i, m in enumerate(self.M):
-            if np.isnan(m):
-                ngtm[i] = np.nan
-            else:
-                # Set up new grid with 4097 steps from m to M=17
-                M_new, dlnM = np.linspace(np.log(m), max_M, 4097, retstep=True)
-                mf_new = mf(M_new)
+        #Calculate the cumulative integral (backwards) of mass_function (Adding on the upper integral)
+        ngtm = intg.cumtrapz(mass_function[::-1], dx=np.log(M[1]) - np.log(M[0]), initial=0)[::-1] + int_upper
 
-                ngtm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
+        #We need to set ngtm back in the original length vector with nans where they were originally
+        if len(ngtm) < len(self.M):
+            ngtm_temp = np.zeros_like(self.dndlnm)
+            ngtm_temp[:] = np.nan
+            ngtm_temp[np.logical_not(np.isnan(self.dndlnm))] = ngtm
+            ngtm = ngtm_temp
 
         return ngtm
 
@@ -1005,33 +1051,6 @@ class Perturbations(object):
         try:
             return self.__ngtm
         except:
-            # Initialize the function
-#            self.__ngtm = np.zeros_like(self.dndlnm)
-#
-#            # set M and mass_function within computed range
-#            M = self.M[np.logical_not(np.isnan(self.dndlnm))]
-#            mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
-#
-#            # Interpolate the mass_function - this is in log-log space.
-#            mf = spline(np.log(M), mass_function, k=1)
-#
-#            # Define max_M as either 18 or the maximum set by user
-#            max_M = np.log(np.max([10 ** 18, M[-1]]))
-#
-#            for i, m in enumerate(self.M):
-#                if np.isnan(m):
-#                    self.__ngtm[i] = np.nan
-#                else:
-#                    # Set up new grid with 4097 steps from m to M=17
-#                    M_new, dlnM = np.linspace(np.log(m), max_M, 4097, retstep=True)
-#                    mf_new = mf(M_new)
-#
-#                    self.__ngtm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
-#
-#            #Here we add a correction for high-z for Tinker, by Behroozi.
-#            if self.mf_fit == 'Behroozi':
-#                pivot_factor = 0.144 / (1 + np.exp(14.79 * (1 / (1 + self.z)) - 0.213))
-#                self.__ngtm = 10 ** (pivot_factor * (self.M / (10 ** 11.5)) ** (0.5 / (1 + np.exp(6.5 * (1 / (1 + self.z))))) + np.log10(self.__ngtm))
             self.__ngtm = self._ngtm()
             return self.__ngtm
 
@@ -1050,30 +1069,26 @@ class Perturbations(object):
         try:
             return self.__mgtm
         except:
-            # Initialize the function
-            self.__mgtm = np.zeros_like(self.dndlnm)
-
-            # set M and mass_function within computed range
             M = self.M[np.logical_not(np.isnan(self.dndlnm))]
-            mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
+            mass_function = self.dndlnm[np.logical_not(np.isnan(self.dndlnm))]
 
-            # Interpolate the mass_function - this is in log-log space.
-            mf = spline(np.log(M), mass_function, k=1)
+            #Calculate the mass function (and its integral) from the highest M up to 10**18
+            if M[-1] < 10 ** 18:
+                m_upper, mf = self._upper_ngtm(M, np.log(mass_function), M[-1] < self.M[-1])
+                int_upper = intg.simps(np.exp(mf + m_upper) , dx=m_upper[2] - m_upper[1], even='first')
+            else:
+                int_upper = 0
 
-            # Define max_M as either 18 or the maximum set by user
-            max_M = np.log(np.max([10 ** 17, M[-1]]))
-
-            for i, m in enumerate(self.M):
-                if np.isnan(m):
-                    self.__mgtm[i] = np.nan
-                else:
-                    # Set up new grid with 4097 steps from m to M=17
-                    M_new, dlnM = np.linspace(np.log10(m), max_M, 4097, retstep=True)
-                    mf_new = mf(M_new)
-                    self.__mgtm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
+            #Calculate the cumulative integral (backwards) of mass_function (Adding on the upper integral)
+            self.__mgtm = intg.cumtrapz(mass_function[::-1] * M[::-1], dx=np.log(M[1]) - np.log(M[0]), initial=0)[::-1] + int_upper
 
+            #We need to set ngtm back in the original length vector with nans where they were originally
+            if len(self.__mgtm) < len(self.M):
+                mgtm_temp = np.zeros_like(self.dndlnm)
+                mgtm_temp[:] = np.nan
+                mgtm_temp[np.logical_not(np.isnan(self.dndlnm))] = self.__mgtm
+                self.__mgtm = mgtm_temp
             return self.__mgtm
-
     @mgtm.deleter
     def mgtm(self):
         try:
@@ -1089,27 +1104,28 @@ class Perturbations(object):
         try:
             return self.__nltm
         except:
-            # Initialize the function
-            self.__nltm = np.zeros_like(self.dndlnm)
-
             # set M and mass_function within computed range
             M = self.M[np.logical_not(np.isnan(self.dndlnm))]
-            mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
+            mass_function = self.dndlnm[np.logical_not(np.isnan(self.dndlnm))]
 
-            # Interpolate the mass_function - this is in log-log space.
-            mf = spline(np.log(M), mass_function, k=3)
+            #Calculate the mass function (and its integral) from the highest M up to 10**18
+            if M[-1] < 10 ** 18:
+                m_lower, mf = self._lower_ngtm(M, np.log(mass_function), M[0] > self.M[0])
 
-            # Define min_M as either 3 or the minimum set by user
-            min_M = np.log(np.min([10 ** 3, M[0]]))
+                int_lower = intg.simps(np.exp(mf), dx=m_lower[2] - m_lower[1], even='first')
+            else:
+                int_lower = 0
+
+            #Calculate the cumulative integral (backwards) of mass_function (Adding on the upper integral)
+            self.__nltm = intg.cumtrapz(mass_function, dx=np.log(M[1]) - np.log(M[0]), initial=0) + int_lower
+
+            #We need to set ngtm back in the original length vector with nans where they were originally
+            if len(self.__nltm) < len(self.M):
+                nltm_temp = np.zeros_like(self.dndlnm)
+                nltm_temp[:] = np.nan
+                nltm_temp[np.logical_not(np.isnan(self.dndlnm))] = self.__nltm
+                self.__nltm = nltm_temp
 
-            for i, m in enumerate(self.M):
-                if np.isnan(m):
-                    self.__nltm[i] = np.nan
-                else:
-                    # Set up new grid with 4097 steps from m to M=17
-                    M_new, dlnM = np.linspace(min_M, np.log10(m), 4097, retstep=True)
-                    mf_new = mf(M_new)
-                    self.__nltm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
 
             return self.__nltm
     @nltm.deleter
@@ -1127,28 +1143,27 @@ class Perturbations(object):
         try:
             return self.__mltm
         except:
-            # Initialize the function
-            self.__mltm = np.zeros_like(self.dndlnm)
-
-            # set M and mass_function within computed range
+            # Set M within calculated range
             M = self.M[np.logical_not(np.isnan(self.dndlnm))]
-            mass_function = np.log(self.dndlnm[np.logical_not(np.isnan(self.dndlnm))])
+            mass_function = self.dndlnm[np.logical_not(np.isnan(self.dndlnm))]
 
+            #Calculate the mass function (and its integral) from the highest M up to 10**18
+            if M[-1] < 10 ** 18:
+                m_lower, mf = self._lower_ngtm(M, np.log(mass_function), M[0] > self.M[0])
 
-            # Interpolate the mass_function - this is in log-log space.
-            mf = spline(np.log(M), mass_function, k=3)
+                int_lower = intg.simps(np.exp(mf + m_lower), dx=m_lower[2] - m_lower[1], even='first')
+            else:
+                int_lower = 0
 
-            # Define max_M as either 17 or the maximum set by user
-            min_M = np.log(np.min([10 ** 3, M[0]]))
+            #Calculate the cumulative integral (backwards) of mass_function (Adding on the upper integral)
+            self.__mltm = intg.cumtrapz(mass_function * M, dx=np.log(M[1]) - np.log(M[0]), initial=0) + int_lower
 
-            for i, m in enumerate(self.M):
-                if np.isnan(m):
-                    self.__mltm[i] = np.nan
-                else:
-                    # Set up new grid with 4097 steps from m to M=17
-                    M_new, dlnM = np.linspace(min_M, np.log10(m), 4097, retstep=True)
-                    mf_new = mf(M_new)
-                    self.__mltm[i] = intg.romb(np.exp(mf_new), dx=dlnM)
+            #We need to set ngtm back in the original length vector with nans where they were originally
+            if len(self.__nltm) < len(self.M):
+                nltm_temp = np.zeros_like(self.dndlnm)
+                nltm_temp[:] = np.nan
+                nltm_temp[np.logical_not(np.isnan(self.dndlnm))] = self.__mltm
+                self.__mltm = nltm_temp
 
             return self.__mltm
 
@@ -1167,266 +1182,7 @@ class Perturbations(object):
         except:
             pass
 
-    def _nufnu_PS(self):
-        """
-        Computes the function nu*f(nu) for the Press-Schechter approach at a given radius.
-        
-        Input R: radius of the top-hat function
-        Output: f_of_nu: the function nu*f(nu) for the PS approach.
-        """
 
-        vfv = np.sqrt(2.0 / np.pi) * (self.cosmo_params['delta_c'] / self.sigma) * np.exp(-0.5 * (self.cosmo_params['delta_c'] / self.sigma) ** 2)
-
-        return vfv
-
-    def _nufnu_ST(self):
-        """
-        Finds the Sheth Tormen vf(v) 
-        
-        Input R: radius of the top-hat function
-        Output: vfv: the Sheth-Tormen mass function fit.
-        """
-
-        nu = self.cosmo_params['delta_c'] / self.sigma
-        a = 0.707
-
-        vfv = 0.3222 * np.sqrt(2.0 * a / np.pi) * nu * np.exp(-(a * nu ** 2) / 2.0) * (1 + (1.0 / (a * nu ** 2)) ** 0.3)
-
-        return vfv
-
-    def _nufnu_Jenkins(self):
-        """
-        Finds the Jenkins empirical vf(v) 
-        
-        Output: vfv: the Jenkins mass function fit.
-        """
-
-        vfv = 0.315 * np.exp(-np.abs(self.lnsigma + 0.61) ** 3.8)
-        # Conditional on sigma range.
-        if self.cut_fit:
-            vfv[np.logical_or(self.lnsigma < -1.2, self.lnsigma > 1.05)] = np.NaN
-
-        return vfv
-
-    def _nufnu_Warren(self):
-        """
-        Finds the Warren empirical vf(v) 
-        
-        Input R: radius of the top-hat function
-        Output: vfv: the Warren mass function fit.
-        """
-
-        vfv = 0.7234 * ((1.0 / self.sigma) ** 1.625 + 0.2538) * np.exp(-1.1982 / self.sigma ** 2)
-
-        if self.cut_fit:
-            vfv[np.logical_or(self.M < 10 ** 10, self.M > 10 ** 15)] = np.NaN
-        return vfv
-
-    def _nufnu_Reed03(self):
-        """
-        Finds the Reed 2003 empirical vf(v) 
-        
-        Input R: radius of the top-hat function
-        Output: vfv: the Reed 2003 mass function fit.
-        
-        NOTE: Only valid from -1.7 < ln sigma^-1 < 0.9
-        """
-
-        ST_Fit = self._nufnu_ST()
-
-        vfv = ST_Fit * np.exp(-0.7 / (self.sigma * np.cosh(2.0 * self.sigma) ** 5))
-
-        if self.cut_fit:
-            vfv[np.logical_or(self.lnsigma < -1.7, self.lnsigma > 0.9)] = np.NaN
-        return vfv
-
-    def _nufnu_Reed07(self):
-        """
-        Finds the Reed 2007 empirical vf(v) 
-        
-        Input R: radius of the top-hat function
-        Output: vfv: the Reed 2003 mass function fit.
-        
-        NOTE: Only valid from -1.7 < ln sigma^-1 < 0.9
-        """
-        nu = self.cosmo_params['delta_c'] / self.sigma
-
-        G_1 = np.exp(-((1.0 / self.sigma - 0.4) ** 2) / (2 * 0.6 ** 2))
-        G_2 = np.exp(-((1.0 / self.sigma - 0.75) ** 2) / (2 * 0.2 ** 2))
-
-        c = 1.08
-        a = 0.764 / c
-        A = 0.3222
-        p = 0.3
-
-        vfv = A * np.sqrt(2.0 * a / np.pi) * (1.0 + (1.0 / (a * nu ** 2)) ** p + 0.6 * G_1 + 0.4 * G_2) * nu * np.exp(-c * a * nu ** 2 / 2.0 - 0.03 * nu ** 0.6 / (self.n_eff + 3) ** 2)
-
-        if self.cut_fit:
-            vfv[np.logical_or(self.lnsigma < -0.5, self.lnsigma > 1.2)] = np.NaN
-
-        return vfv
-
-
-    def _nufnu_Angulo(self):
-
-        vfv = 0.201 * ((2.08 / self.sigma) ** 1.7 + 1) * np.exp(-1.172 / self.sigma ** 2)
-        return vfv
-
-    def _nufnu_Angulo_Bound(self):
-        vfv = 0.265 * ((1.675 / self.sigma) ** 1.9 + 1) * np.exp(-1.4 / self.sigma ** 2)
-        return vfv
-
-    def _nufnu_Tinker(self):
-
-        #The Tinker function is a bit tricky - we use the code from http://cosmo.nyu.edu/~tinker/massfunction/MF_code.tar
-        #to aide us.
-        delta_virs = np.array([200, 300, 400, 600, 800, 1200, 1600, 2400, 3200])
-        A_array = np.array([ 1.858659e-01,
-                            1.995973e-01,
-                            2.115659e-01,
-                            2.184113e-01,
-                            2.480968e-01,
-                            2.546053e-01,
-                            2.600000e-01,
-                            2.600000e-01,
-                            2.600000e-01])
-
-        a_array = np.array([1.466904e+00,
-                            1.521782e+00,
-                            1.559186e+00,
-                            1.614585e+00,
-                            1.869936e+00,
-                            2.128056e+00,
-                            2.301275e+00,
-                            2.529241e+00,
-                            2.661983e+00])
-
-        b_array = np.array([2.571104e+00 ,
-                            2.254217e+00,
-                            2.048674e+00,
-                            1.869559e+00,
-                            1.588649e+00,
-                            1.507134e+00,
-                            1.464374e+00,
-                            1.436827e+00,
-                            1.405210e+00])
-
-        c_array = np.array([1.193958e+00,
-                            1.270316e+00,
-                            1.335191e+00,
-                            1.446266e+00,
-                            1.581345e+00,
-                            1.795050e+00,
-                            1.965613e+00,
-                            2.237466e+00,
-                            2.439729e+00])
-        A_func = spline(delta_virs, A_array)
-        a_func = spline(delta_virs, a_array)
-        b_func = spline(delta_virs, b_array)
-        c_func = spline(delta_virs, c_array)
-
-        A_0 = A_func(self.delta_vir)
-        a_0 = a_func(self.delta_vir)
-        b_0 = b_func(self.delta_vir)
-        c_0 = c_func(self.delta_vir)
-
-        A = A_0 * (1 + self.z) ** (-0.14)
-        a = a_0 * (1 + self.z) ** (-0.06)
-        alpha = np.exp(-(0.75 / np.log(self.delta_vir / 75)) ** 1.2)
-        b = b_0 * (1 + self.z) ** (-alpha)
-        c = c_0
-
-
-        vfv = A * ((self.sigma / b) ** (-a) + 1) * np.exp(-c / self.sigma ** 2)
-
-        if self.cut_fit:
-            vfv[np.logical_or(self.lnsigma < -0.6 , self.lnsigma > 0.4)] = np.nan
-
-        return vfv
-
-    def _watson_gamma(self):
-        C = np.exp(0.023 * (self.delta_vir / 178 - 1))
-        d = -0.456 * cosmography.omegam_z(self.z, self.cosmo_params['omegam'], self.cosmo_params['omegav'], self.cosmo_params['omegak']) - 0.139
-        p = 0.072
-        q = 2.13
-
-        return C * (self.delta_vir / 178) ** d * np.exp(p * (1 - self.delta_vir / 178) / self.sigma ** q)
-
-
-    def _nufnu_Watson_FoF(self):
-        vfv = 0.282 * ((1.406 / self.sigma) ** 2.163 + 1) * np.exp(-1.21 / self.sigma ** 2)
-        if self.cut_fit:
-            vfv[np.logical_or(self.lnsigma < -0.55 , self.lnsigma > 1.31)] = np.NaN
-        return vfv
-
-    def _nufnu_Watson(self):
-
-        if self.z == 0:
-            A = 0.194
-            alpha = 2.267
-            beta = 1.805
-            gamma = 1.287
-        elif self.z > 6:
-            A = 0.563
-            alpha = 0.874
-            beta = 3.810
-            gamma = 1.453
-        else:
-            A = cosmography.omegam_z(self.z, self.cosmo_params['omegam'], self.cosmo_params['omegav'], self.cosmo_params['omegak']) * (1.097 * (1 + self.z) ** (-3.216) + 0.074)
-            alpha = cosmography.omegam_z(self.z, self.cosmo_params['omegam'], self.cosmo_params['omegav'], self.cosmo_params['omegak']) * (3.136 * (1 + self.z) ** (-3.058) + 2.349)
-            beta = cosmography.omegam_z(self.z, self.cosmo_params['omegam'], self.cosmo_params['omegav'], self.cosmo_params['omegak']) * (5.907 * (1 + self.z) ** (-3.599) + 2.344)
-            gamma = 1.318
-
-        vfv = self._watson_gamma() * A * ((beta / self.sigma) ** alpha + 1) * np.exp(-gamma / self.sigma ** 2)
-
-        if self.cut_fit:
-            vfv[np.logical_or(self.lnsigma < -0.55, self.lnsigma > 1.05)] = np.NaN
-
-        return vfv
-
-    def _nufnu_Crocce(self):
-
-        A = 0.58 * (1 + self.z) ** (-0.13)
-        a = 1.37 * (1 + self.z) ** (-0.15)
-        b = 0.3 * (1 + self.z) ** (-0.084)
-        c = 1.036 * (1 + self.z) ** (-0.024)
-
-        vfv = A * (self.sigma ** (-a) + b) * np.exp(-c / self.sigma ** 2)
-        return vfv
-
-    def _nufnu_Courtin(self):
-        A = 0.348
-        a = 0.695
-        p = 0.1
-        d_c = self.cosmo_params['delta_c']  # Note for WMAP5 they find delta_c = 1.673
-
-        vfv = A * np.sqrt(2 * a / np.pi) * (d_c / self.sigma) * (1 + (d_c / (self.sigma * np.sqrt(a))) ** (-2 * p)) * np.exp(-d_c ** 2 * a / (2 * self.sigma ** 2))
-        return vfv
-
-    def _nufnu_Bhattacharya(self):
-        A = 0.333 * (1 + self.z) ** -0.11
-        a = 0.788 * (1 + self.z) ** -0.01
-        p = 0.807
-        q = 1.795
-
-        nu = self.cosmo_params['delta_c'] / self.sigma
-
-        vfv = A * np.sqrt(2.0 / np.pi) * np.exp(-(a * nu ** 2) / 2.0) * (1 + (1.0 / (a * nu ** 2)) ** p) * (nu * np.sqrt(a)) ** q
-        if self.cut_fit:
-            vfv[np.logical_or(self.M < 6 * 10 ** 11, self.M > 3 * 10 ** 15)] = np.NaN
-
-        return vfv
-
-    def _nufnu_user_model(self):
-        """
-        Calculates vfv based on a user-input model.
-        """
-        from scitools.StringFunction import StringFunction
-
-        f = StringFunction(self.user_fit, globals=globals())
-
-
-        return f(self.sigma)
 
 
 
